	PAGE	60,130
	TITLE	DHPRTSCR - TSR PrtScr driver to fix IBM ROM BIOS LFCR problem
;
;  Resident PrtScr program that makes Interrupt 5 (PrtScr) output CRLF for
;  new line, instead of LFCR, which confuses some printers.
;  By Dave Hardy and Keith Petersen.
;  Surprise!  Released to Public Domain 01/18/88  Keep your money.
;
;    Besides being an almost trivial program, this is a reasonable
;  example of a simple TSR (Terminate and Stay Resident) process.
;    When run, this program patches itself in as the interrupt 5
;  handler, which is the part of the BIOS that dumps the contents 
;  of the screen to the printer when the Shift+PrtSc keys are pressed.
;    The only difference between this handler and the one built into the
;  11/08/82 IBM PC/XT BIOS is that this one sends a CR, then a LF to the
;  printer at the end of each screen line, while the IBM BIOS' handler sends
;  a LF, then a CR, which causes some older printers to forget to linefeed.
;
;  To assemble this file:
;    MASM DHPRTSCR;			<--- to make an .OBJ file
;    LINK DHPRTSCR;			<--- to link .OBJ file to make .EXE
;    EXE2BIN DHPRTSCR DHPRTSCR.COM	<--- to convert to simple .COM file
;
;  Define a carriage return and a linefeed
CR	EQU	0DH
LF	EQU	0AH
;
;  Tell MASM (because it is basically a stupid compiler) that this is code
CODE	SEGMENT
;  Tell MASM to assume that DOS will set up registers like this for execution
	ASSUME DS:CODE, SS:CODE ,CS:CODE ,ES:CODE
;  Tell MASM that this is to be a .COM file, not a relocatable .EXE file
	ORG	100H
;
START:	JMP	INSTAL	;Jump past resident stuff to disposable install code
;
;  Interrupt handler
;  This is the actual PrtSc handler whose address is patched into INT 5
PRTSCR:	STI				;Disable interrupts
	PUSH	DS			;Save everything, since this is BIOS 
	PUSH	AX			;  and we don't want to screw up caller
	PUSH	BX
	PUSH	CX
	PUSH	DX
;  First get control of the printer
	MOV	AX,0050H		;Look at printer status (0050:0000)
	MOV	DS,AX
	CMP	BYTE PTR DS:[0000],01	;See if printer already in use
	JZ	PXIT			;  by some other process.  Exit if so.
	MOV	BYTE PTR DS:[0000],01	;Else mark it as being used
;  Then set up to read and print each character of screen
	MOV	AH,0FH		;Get current screen mode
	INT	10H		;Returns AL=mode, AH=cols per line, BH=page#
	MOV	CL,AH		;Set up CX with screen size (40|80 x 25)
	MOV	CH,25
	CALL	PCRLF		;Print a new line on the list device
	PUSH	CX		;Save screen size for later
	MOV	AH,03H		;Get current cursor position
	INT	10H
	POP	CX		;Restore screen size
	PUSH	DX		;Save cursor position for later
	XOR	DX,DX		;Set for cursor position of 0,0
;  This is the loop that actually reads and prints each character
PRNSCN:	MOV	AH,02H		;Set cursor position to DH,DL
	INT	10H
	MOV	AH,08H		;Read character at position DH,DL into AL
	INT	10H
	OR	AL,AL		;See if valid character
	JNZ	NOTSPC		;Jump if yes,
	MOV	AL,20H		;Else change it to a space character
;
NOTSPC:	PUSH	DX		;Save cursor position
	XOR	DX,DX		;Send character in AL to printer 1 (=DX)
	XOR	AH,AH		;	AH=0 to write character to printer
	INT	17H		;	Do the interrupt
	POP	DX		;Restore cursor position
	TEST	AH,25H		;Check for error returned from printer
	JNZ	PERROR		;Jump if error
	INC	DL		;Else point to next column to print
	CMP	CL,DL		;See if at end of line
	JNZ	PRNSCN		;Jump if not
	XOR	DL,DL		;Else back to column 0
	MOV	AH,DL
	PUSH	DX		;Save new cursor position
	CALL	PCRLF		;Send a CRLF to the list device
	POP	DX		;Restore cursor position
	INC	DH		;Advance to next line
	CMP	CH,DH		;See if finished
	JNZ	PRNSCN		;Jump if not
	POP	DX		;Else restore original cursor position
	MOV	AH,02
	INT	10H
	MOV	BYTE PTR DS:[0000],00	;Flag printer as idle (release it)
	JMP	PXIT			;Restore all, IRET to system or whatever
;
PERROR:	POP	DX			;Restore cursor to original position
	MOV	AH,02
	INT	10H
	MOV	BYTE PTR DS:[0000],0FFH	;Set printer flag to say error
;
PXIT:	POP	DX			;Restore everything as before entry
	POP	CX
	POP	BX
	POP	AX
	POP	DS
	IRET				;Return to system or whatever
;
;  Subroutine to send a CRLF (instead of a LFCR) to the list device
PCRLF:	XOR	DX,DX
	XOR	AH,AH
	MOV	AL,CR		;Note that we send the CR first,
	INT	17H
	XOR	AH,AH
	MOV	AL,LF		;  then we send the LF.
	INT	17H
	RET
;
; END OF RESIDENT CODE  (Code to this point remains resident upon exit)
; START OF DISPOSABLE CODE  (Code from here on is released upon exit)
;
;  The install routines
INSTAL:	LEA	DX,SGNMSG	;Sign on and tell user what we're up to
	MOV	AH,9
	INT	21H
;
	MOV	AX,2505H	;Set vector 5 (PrtSc) to DS:DX (=PRTSCR here)
	LEA	DX,PRTSCR
	INT	21H
;
	LEA	DX,INSTAL	;Set up for TSR.  Get # of paragraphs into DX
	ADD	DX,0FH		;(Actually, this saves 15, should to 2)
	MOV	CL,4
	SHR	DX,CL
	MOV	AH,31H		;DOS INT 21H Function 31H (TSR)
	MOV	AL,0		;Return result code of 0 (=OK) for errorlevel
	INT	21H		;Note that we could have used INT 27H instead...
	INT	20H		;Exit back to DOS
;
;
SGNMSG	DB	'DHPRTSCR - Resident PrtSc replacement to fix CRLF problem.'
	DB	CR,LF,'$'
;  Insult the curious (Actually, MASM does this all by itself)
	DB	CR,LF,'Nosey, aren',27H,'t you?'
;  Tell MASM that the code segment ends here
CODE	ENDS
;  Tell MASM that program ends here, and that execution begins at label START
	END	START
